#include <iostream>
#include <assert.h>
#include <SDL/SDL.h>
#include <math.h>
#include <iostream>
#include <vector>

#include "glinc.h"
#include "Math.h"
#include "Vector3.h"

using namespace Math;
using namespace std;

static SDL_Surface *surface;
static float xrot = 0.0f;
static float yrot = 0.0f;

struct point {
	Vector3f coord;
	Vector3f color;
};

static int setup_sdl()
{
	// set up sdl
	std::cout << "initializing SDL" << std::endl;
	atexit(SDL_Quit);
	SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER);

    int ret = SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 8 );                        // In order to use SDL_OPENGLBLIT we have to
    ret = SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 8 );                          // set GL attributes first
    ret = SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 8 );
    ret = SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 24 );
    ret = SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );                        // colors and doublebuffering

    surface = SDL_SetVideoMode(800, 800, 32, SDL_OPENGL|SDL_RESIZABLE);
    if (!surface) {
        std::cout << "error creating sdl/gl window\n" << std::endl;
        return -1;
    }

	SDL_FillRect(surface, NULL, SDL_MapRGBA(surface->format,0,0,0,0));

	SDL_WM_SetCaption("gltest","gltest");
}

void InitGLExt()
{
#if WITH_GLEW
    GLenum err = glewInit();
    if (err != GLEW_OK) {
        fprintf(stderr, "error initializing GLEW: %s\n", glewGetErrorString(err));
        assert(0);
    }

    printf("GLEW version %s\n", glewGetString(GLEW_VERSION));

	// print some stuff
	if (GLEW_ARB_vertex_program) {
		printf("ARB_vertex_program available\n");
	}
	if (GLEW_ARB_fragment_program) {
		printf("ARB_fragment_program available\n");
	}

#endif
}

static void frame_start()
{
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glViewport(0, 0, (GLsizei)800, (GLsizei)800);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
//	gluOrtho2D(0, 800, 600, 0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glRotatef(xrot, 0, 1.0f, 0);
	glRotatef(yrot, 1.0f, 0, 0);
}

static void frame_end()
{
	glFlush();
	glFinish();

	SDL_GL_SwapBuffers();
}

static void draw_rot_triangle()
{
	glPushMatrix();
	glBegin(GL_TRIANGLES);                              // Begin Drawing Triangles
	glColor3f (1.f, 0.f, 0.f);                      // 1st Color (Red) ...
	glVertex3f( 0.0f, 1.0f, 0.0f);              // ...For The 1st Vertex
	glColor3f (0.f, 1.f, 0.f);                      // 2nd Color (Green) ...
	glVertex3f(-1.0f,-1.0f, 0.0f);              // ...For The 2nd Vertex
	glColor3f (0.f, 0.f, 1.f);                      // And 3rd Color (Blue) ...
	glVertex3f( 1.0f,-1.0f, 0.0f);              // ...For The 3rd Vertex
	glEnd();                                            // Done Drawing Triangles
	glPopMatrix();
}

static void draw_rot_quad()
{
	glPushMatrix();
	glBegin(GL_QUADS);                              // Begin Drawing quad
	glColor3f (1.f, 0.f, 0.f);                      // 1st Color (Red) ...
	glVertex3f( -1.0f, 1.0f, 0.0f);              // ...For The 1st Vertex
	glColor3f (0.f, 1.f, 0.f);                      // 2nd Color (Green) ...
	glVertex3f(1.0f, 1.0f, 0.0f);              // ...For The 2nd Vertex
	glColor3f (0.f, 0.f, 1.f);                      // And 3rd Color (Blue) ...
	glVertex3f(1.0f,-1.0f, 0.0f);              // ...For The 3rd Vertex
	glColor3f (1.f, 0.f, 1.f);                      // And 4rd Color (Blue) ...
	glVertex3f(-1.0f,-1.0f, 0.0f);              // ...For The 4rd Vertex
	glEnd();                                            // Done Drawing Triangles
	glPopMatrix();
}

void draw_sphere()
{

	glPushMatrix();
	glBegin(GL_LINES);
	glColor3f(1.0, 0, 0);
	glVertex3f(0,0,0);
	glVertex3f(1,0,0);
	glColor3f(0, 1.0, 0);
	glVertex3f(0,0,0);
	glVertex3f(0,1,0);
	glColor3f(0, 0, 1.0);
	glVertex3f(0,0,0);
	glVertex3f(0,0,1);
	glEnd();

#if 1
	glBegin(GL_QUADS);
	glColor3f (1.f, 0.f, 0.f);                      // 1st Color (Red) ...
	glVertex3f( -1.0f, 1.0f, 0.0f);              // ...For The 1st Vertex
	glColor3f (0.f, 1.f, 0.f);                      // 2nd Color (Green) ...
	glVertex3f(1.0f, 1.0f, 0.0f);              // ...For The 2nd Vertex
	glColor3f (0.f, 0.f, 1.f);                      // And 3rd Color (Blue) ...
	glVertex3f(1.0f,-1.0f, 0.0f);              // ...For The 3rd Vertex
	glColor3f (1.f, 0.f, 1.f);                      // And 4rd Color (Blue) ...
	glVertex3f(-1.0f,-1.0f, 0.0f);              // ...For The 4rd Vertex
	glEnd();
#endif

	glBegin(GL_POINTS);
	glColor3f(1.0, 1.0, 1.0);

#define PI (3.14159f)

#if 0
	for (float a = 0.0; a <= 2*PI; a += PI/100) {
		float x = sinf(a);
		float y = cosf(a);

		glVertex3f(0,0,0);
		glVertex3f(y,x,0);
	}
#endif
#if 1
#define COUNT 500
	static bool gen = false;
	static std::vector<point> points;
	typedef std::vector<point>::iterator PointIterator;

	if (!gen) {
		// spherical coordinates
		float r = 1.0f;
		float theta;
		float phi;

		for (float i = 0; i <= 1.0; i += 1.0/COUNT) {
			for (float j = 0; j <= 1.0; j += 1.0/COUNT) {
#if 1
				float u = drand48();
				float v = drand48();
#else
				float u = (float)i;
				float v = (float)j;
#endif
				phi = 2 * PI * u;
				theta = acos(2 * v - 1);
				float x = r * sin(theta) * cos(phi);
				float y = r * sin(theta) * sin(phi);
				float z = r * cos(theta);

				point p;
				p.coord = Vector3f(x, y, z);

#if 0
				float c = Dot(p.coord, Vector3f(1, 1, 1));
#else
				float c = 1.0f;
#endif

				p.color = Vector3f(c, c, c);
		
	//			std::cout << c << std::endl;

				points.push_back(p);
			}
		}

		gen = true;
	}

	for (PointIterator i = points.begin(); i != points.end(); i++) {
		if ((*i).color.getx() > 0) {
			glColor3f((*i).color.getx(), (*i).color.gety(), (*i).color.getz());
			glVertex3f((*i).coord.gety(), (*i).coord.getx(), (*i).coord.getz());
		}
	}

#endif

	glEnd();
	glPopMatrix();
}

int main(int argc, char **argv)
{
	setup_sdl();
	InitGLExt();

	for (;;) {

		SDL_Event event;
		if (SDL_PollEvent(&event)) {
			switch (event.type) {
				case SDL_QUIT:
					goto done;
				case SDL_MOUSEMOTION:
//					printf("%d %d %d.%d\n", event.motion.which, event.motion.state, event.motion.xrel, event.motion.yrel);
					if (event.motion.state == 1) {
						xrot -= event.motion.xrel * 0.1f;
						yrot -= event.motion.yrel * 0.1f;
					}
					break;
			}
		}

		frame_start();

//		draw_rot_triangle();
//		draw_rot_quad();

		draw_sphere();

		frame_end();
	}
done:

	return 0;
}
